%%%%%%%%%%%%%%%%%%%%%%%%%
% IMPORT AND CONVERT IMAGE

im=imread('SampleRipplesBrest3Crop.jpg');% Reads image from file. If image is not in current folder specify path.

im=rgb2gray(im);% Converts any truecolor image RGB image to a grayscale intensity image (eliminates the hue and saturation information while retaining the luminance).


%%%%%%%%%%%%%%%%%%%%%%%%%
% DERIVE RIPPLE PARAMETERS

[normim,mask] = ridgesegment (im, blksze, thresh)
%              im     - Image
%              blksze - Block size (subset of image pixels) over which the isotropic standard deviation is determined.
%              thresh - Threshold of standard deviation to delineate if a block is classified as rippled. 
%
%              You may want to experiment with various values for blksze and thresh to determine how
%              they affect the mask. This is just an initial masking for normalization purposes so it does
%              not have to be perfect.
%
[orientim, reliability] = ridgeorient(normim, gradientsigma, blocksigma, orientsmoothsigma);
%              normim            - Normalized (zero mean) image.
%              gradientsigma     - Sigma of the derivative of Gaussian used to compute image gradients.
%              blocksigma        - Sigma of the Gaussian weighting used to sum the gradient moments.
%              orientsmoothsigma - Sigma of the Gaussian used to smooth the final orientation vector field.
%
%              You may want to experiment with various values for gradientsigma, blocksigma, orientsmoothsigma to determine how
%              they affect the orientation and reliability results. This is just an initial masking for normalization purposes so it does
%              not have to be perfect.
%
reliability(reliability>0)=1;
rmask=logical(reliability); % generates a logical mask of rippled vs non rippled areas based upon pixel variability perpendicular to ripple orientation relative to pixel variability parallel to ripple orientation.
%
%[freq] = ridgefreq(normim, rmask, orientim, blksze, windsze, minWaveLength, maxWaveLength);% Assigns frequency value returend from blksze x blksze block to all pixels in block.
%[freq] = rf2(normim, rmask, orientim, blksze, windsze, minWaveLength, maxWaveLength);% Assigns frequency values returend from blksze x blksze block to central pixel. Highly processor intensive.
[freq] = rf3(normim, rmask, orientim, blksze, windsze, minWaveLength, maxWaveLength);% Assigns frequency values returend from blksze x blksze block block to central 1/8 (rounded to nearest integer) size block.
%             normim   - Normalized (zero mean) image.
%             rmask    - Logical mask defining rippled vs non rippled regions of image.
%             orientim - Ripple orientation image (obtained from RIDGORIENT).
%             blksze   - Size of image block in pixels to use to determine frequency (must be an odd integer). 
%             windsze  - Window length in pixels used to identify ripple peaks.
%             minWaveLength,  maxWaveLength - Minimum and maximum ridge wavelengths, in pixels, considered acceptable.
%
%             Note that Ridgefreq, rf2, and rf3 may call freqest or freqest_AS to determine
%             ripple wavelength. The file frequest determines ripple wavelenght via identification
%             of local peaks (within a window of 'windsze' in a profile perpenduclar to ripple
%             orientation. The file frequest_AS determines ripple wavelength via identification of
%             zero crossings in the profile and generally gives better results.
%
%             I want to use a 2D fft calculation here to determine wavelength in a future iteration.
%             
%%%%%%%%%%%%%%%%%%%%%%%%%
% IDENTIFY RIPPLE DEFECTS

newim=ridgefilter(normim, orientim, freqim, kx, ky, showfilter);
%             normim       - Image to be processed.
%             orientim     - Ridge orientation image (obtained from RIDGEORIENT).
%             freqim       - Ridge frequency image (obtained from RIDGEFREQ, rf2, or rf3).
%             kx, ky       - Scale factors specifying the filter sigma relative
%                            to the wavelength of the filter.  This is done so
%                            that the shapes of the filters are invariant to the
%                            scale.  kx controls the sigma in the x direction
%                            which is along the filter, and hence controls the
%                            bandwidth of the filter.  ky controls the sigma
%                            across the filter and hence controls the
%                            orientational selectivity of the filter. 
%             showfilter   - An optional flag 0/1.  When set an image of the
%                            largest scale filter is displayed for inspection.
%
% Binarize the filtered image.
binim=newim>0; 

% Thin binarized lines to create a map of ripple crest with a single pixel wide line.
a=bwmorph(binim, 'thin', Inf);
a=bwmorph(a, 'fill');
a=bwmorph(a, 'thin', Inf);
a=bwmorph(a, 'spur');
a=bwmorph(a, 'thin', Inf);
a=bwmorph(a, 'spur');
a=bwmorph(a, 'clean');

% Isolate bifurcations.
c=a;
m=find(a==1);
for g=1:length(m);
    [X, Y] = ind2sub([size(a)],m(g));
    b=a((X-1):(X+1),(Y-1):(Y+1));
    if sum(b(:))==4;
        c(X,Y)=1;
    else
        c(X,Y)=0;
    end
end
c=bwmorph(c,'shrink', Inf);
bifurcations=bwmorph(c,'dilate');

% Isolate terminations.
d=a;
m=find(a==1);
for g=1:length(m);
    [X, Y] = ind2sub([size(d)],m(g));
    b=a((X-1):(X+1),(Y-1):(Y+1));
    if sum(b(:))==2;
        d(X,Y)=1;
    else
        d(X,Y)=0;
    end
end
d=bwmorph(d,'shrink', Inf);
terminations=bwmorph(d,'dilate'); 


% Remove “edge” terminations on discrete ripple features (e.g. sorted bedforms)

emask=abs(rmask-1);
fmask=bwmorph(emask,'dilate',3);
h=logical(a+fmask);
d=a;
m=find(a==1);
for g=1:length(m);
    [X, Y] = ind2sub([size(a)],m(g));
    b=h((X-1):(X+1),(Y-1):(Y+1));
    if sum(b(:))==2;
        d(X,Y)=1;
    else
        d(X,Y)=0;
    end
end
d=bwmorph(d,'shrink', Inf);
edgeterminations=bwmorph(d,'dilate');




